@@ -2,13 +2,14 @@ var searchData=
 [
   ['maps_2eh',['Maps.h',['../Maps_8h.html',1,'']]],
   ['mapsutil_2eh',['MapsUtil.h',['../MapsUtil_8h.html',1,'']]],
+  ['mask_2eh',['Mask.h',['../Mask_8h.html',1,'']]],
   ['mat_2eh',['Mat.h',['../Mat_8h.html',1,'']]],
   ['mat3_2eh',['Mat3.h',['../Mat3_8h.html',1,'']]],
   ['mat4_2eh',['Mat4.h',['../Mat4_8h.html',1,'']]],
   ['math_2eh',['Math.h',['../Math_8h.html',1,'']]],
   ['math_2etxt',['math.txt',['../math_8txt.html',1,'']]],
   ['meshtovolume_2eh',['MeshToVolume.h',['../MeshToVolume_8h.html',1,'']]],
-  ['metadata_2eh',['Metadata.h',['../Metadata_8h.html',1,'(Global Namespace)'],['../metadata_2Metadata_8h.html',1,'(Global Namespace)']]],
+  ['metadata_2eh',['Metadata.h',['../Metadata_8h.html',1,'']]],
   ['metamap_2eh',['MetaMap.h',['../MetaMap_8h.html',1,'']]],
   ['morphology_2eh',['Morphology.h',['../Morphology_8h.html',1,'']]],
   ['multiresgrid_2eh',['MultiResGrid.h',['../MultiResGrid_8h.html',1,'']]]
